/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002 - 2004
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package EDU.utwente.CoCompose;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.reflect.Array;
import java.util.Properties;

import javax.swing.UIManager;
import EDU.utwente.CoCompose.gui.MainFrame;

/**
 * Main application.
 *
 * @author Dennis Wagelaar
 * @version 0.1.20, 4 April 2004
 */

public class CoCompose
{
  //////////////////////////////////////////////////////////////////////////////
  // Constants
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Application major version.
   */
  public static final String MAJOR_VERSION = "0";
  /**
   * Application minor version.
   */
  public static final String MINOR_VERSION = "1";
  /**
   * Application minor version suffix.
   */
  public static final String MINOR_SUFFIX = "20";
  /**
   * Application date.
   */
  public static final String DATE = "4 April 2004";
  // super-languages config file
  private static final String SUPER_LANG_FILE = "superlang.properties"; 
  
  //////////////////////////////////////////////////////////////////////////////
  // Attributes
  //////////////////////////////////////////////////////////////////////////////

  boolean packFrame = false;
  // lookup table of superclass target languages
  private static Properties superLanguages_ = new Properties();

  //////////////////////////////////////////////////////////////////////////////
  // Constructors
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs the application.
   */
  public CoCompose()
  {
    super();
    MainFrame frame = new MainFrame();
    //Validate frames that have preset sizes
    //Pack frames that have useful preferred size info, e.g. from their layout
    if (packFrame)
      frame.pack();
    else
      frame.validate();
    frame.setVisible(true);
  } //CoCompose

  //////////////////////////////////////////////////////////////////////////////
  // Public methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Main method.
   *
   * @param args Application arguments array.
   */
  public static void main(String[] args)
  {
    UIManager.LookAndFeelInfo[] lookandfeels;
    int index = 0;

    try
    {
      if (Array.getLength(args) > 0)
      {
        lookandfeels = UIManager.getInstalledLookAndFeels();
        while (index < Array.getLength(lookandfeels))
        {
          if (lookandfeels[index].getName().equals(args[0]))
          {
            UIManager.setLookAndFeel(lookandfeels[index].getClassName());
            break;
          }
          index++;
        }
      }
      /* Don't apply default JBuilder behaviour
      else
      {
        UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
      }
      */
    }
    catch(Exception e)
    {
    }
    try
    {
      superLanguages_.load(new FileInputStream(new File(SUPER_LANG_FILE)));
    }
    catch (IOException ioe)
    { // if no file available, then just don't load
    }
    new CoCompose();
    try
    {
      superLanguages_.store(new FileOutputStream(new File(SUPER_LANG_FILE)), "super-class target languages");
    }
    catch (IOException ioe)
    {
      System.out.println("Could not write " + SUPER_LANG_FILE + " file!");
      ioe.printStackTrace();
    }
  } //main
  
  /**
   * Sets the super-language for a target language (e.g. AspectJ has
   * Java as super-language).
   * @param subLanguage the language to set a super-language for
   * @param superLanguage the super-language
   */
  public static void setSuperLanguage(String subLanguage, String superLanguage)
  {
    superLanguages_.setProperty(subLanguage, superLanguage);
  } //setSuperLanguage
  
  /**
   * Gets the super-language for a target language (e.g. AspectJ has
   * Java as super-language).
   * @param subLanguage the language to get a super-language for
   * @return the super-language or null
   */
  public static String getSuperLanguage(String subLanguage)
  {
    return superLanguages_.getProperty(subLanguage);
  } //getSuperLanguage
  
  /**
   * Checks whether superLanguage is a super-language of subLanguage,
   * whether direct or indirect.
   * @param subLanguage the language to check
   * @param superLanguage the super-language to check for
   * @return True if superLanguage is a super-language of subLanguage
   */
  public static boolean isSuperLanguage(String subLanguage, String superLanguage)
  {
    String language = superLanguages_.getProperty(subLanguage);
    if (language == null)
      return false;
    return language.equals(superLanguage) || isSuperLanguage(language, superLanguage);
  } //isSuperLanguage
} //class CoCompose
