/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002, 2003
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package EDU.utwente.CoCompose.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.File;

import javax.swing.*;
import javax.swing.border.*;
import java.util.*;
import EDU.utwente.CoCompose.lang.*;
import EDU.utwente.CoCompose.util.DirFileFilter;

import javax.swing.event.*;

/**
 * A dialog window for editing the properties of a Model.
 *
 * @author Dennis Wagelaar
 * @version 0.1.14, 14 October 2003
 */

public class ModelPropertiesDialog extends CoComposeDialog
{
  //////////////////////////////////////////////////////////////////////////////
  // Attributes
  //////////////////////////////////////////////////////////////////////////////

  private Model model_;
  ImageIcon loadIcon = new ImageIcon("images/open.gif");
  JPanel panel1 = new JPanel();
  BorderLayout borderLayout1 = new BorderLayout();
  JPanel mainPanel = new JPanel();
  JPanel buttonPanel = new JPanel();
  JButton okButton = new JButton();
  JPanel targetLangPanel = new JPanel();
  BorderLayout borderLayout2 = new BorderLayout();
  TitledBorder titledBorder1;
  JPanel mainPanel2 = new JPanel();
  BorderLayout borderLayout3 = new BorderLayout();
  JPanel priorityPanel = new JPanel();
  TitledBorder titledBorder2;
  JLabel jLabel1 = new JLabel();
  JTextField langEdit = new JTextField();
  JPanel priorityItemPanel = new JPanel();
  BorderLayout borderLayout4 = new BorderLayout();
  JPanel priorityPanel2 = new JPanel();
  JLabel jLabel2 = new JLabel();
  FlowLayout flowLayout1 = new FlowLayout();
  JTextField langItemEdit = new JTextField();
  JButton cancelButton = new JButton();
  JLabel jLabel3 = new JLabel();
  JTextField formItemEdit = new JTextField();
  FlowLayout flowLayout2 = new FlowLayout();
  JPanel priorityButtonPanel = new JPanel();
  BorderLayout borderLayout5 = new BorderLayout();
  JPanel priorityPanel3 = new JPanel();
  BorderLayout borderLayout6 = new BorderLayout();
  FlowLayout flowLayout3 = new FlowLayout();
  JButton upBtn = new JButton();
  JButton removeBtn = new JButton();
  JButton addBtn = new JButton();
  JButton downBtn = new JButton();
  JPanel priorityListPanel = new JPanel();
  FlowLayout flowLayout4 = new FlowLayout();
  JScrollPane languageScrollPane = new JScrollPane();
  JScrollPane formScrollPane = new JScrollPane();
  JList languageList = new JList();
  JList formList = new JList();
  JLabel jLabel4 = new JLabel();
  JLabel jLabel5 = new JLabel();
  JLabel jLabel6 = new JLabel();
  JButton loadBtn = new JButton();

  //////////////////////////////////////////////////////////////////////////////
  // Constructors
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs a ModelPropertiesDialog for a Model object.
   *
   * @param frame The parent frame.
   * @param title The dialog title.
   * @param modal Shows the dialog in modal mode when true.
   * @param model The Model of which the properties will be edited.
   */
  public ModelPropertiesDialog(Frame frame, String title, boolean modal,
    Model model)
  {
    super(frame, title, modal);
    model_ = model;
    try
    {
      jbInit();
      pack();
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  } //ModelPropertiesDialog(Frame frame, String title, boolean modal)

  /**
   * Constructs a ModelPropertiesDialog.
   * This constructor is only for the JBuilder Designer, do not use it!
   */
  public ModelPropertiesDialog()
  {
    this(null, "", false, new Model());
  } //ModelPropertiesDialog()

  //////////////////////////////////////////////////////////////////////////////
  // Private methods
  //////////////////////////////////////////////////////////////////////////////

  void jbInit() throws Exception
  {
    titledBorder1 = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(148, 145, 140)),"Target language");
    titledBorder2 = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(148, 145, 140)),"Translation priority");
    panel1.setLayout(borderLayout1);
    okButton.setText("OK");
    okButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        okButton_actionPerformed(e);
      }
    });
    mainPanel.setLayout(borderLayout2);
    targetLangPanel.setBorder(titledBorder1);
    targetLangPanel.setLayout(flowLayout1);
    mainPanel2.setLayout(borderLayout3);
    priorityPanel.setBorder(titledBorder2);
    priorityPanel.setLayout(borderLayout4);
    jLabel1.setMaximumSize(new Dimension(900, 17));
    jLabel1.setMinimumSize(new Dimension(200, 17));
    jLabel1.setPreferredSize(new Dimension(100, 17));
    jLabel1.setText("Target language:");
    langEdit.setFont(new java.awt.Font("Dialog", 0, 12));
    langEdit.setPreferredSize(new Dimension(200, 21));
    loadBtn.setPreferredSize(new Dimension(200, 21));
    loadBtn.setText("Load from model");
    loadBtn.setIcon(loadIcon);
    loadBtn.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        loadBtn_actionPerformed(e);
      }
    });
    panel1.setBorder(BorderFactory.createEtchedBorder());
    jLabel2.setText("Target language:");
    jLabel2.setPreferredSize(new Dimension(100, 17));
    jLabel2.setMinimumSize(new Dimension(100, 17));
    jLabel2.setMaximumSize(new Dimension(100, 17));
    langItemEdit.setPreferredSize(new Dimension(200, 21));
    langItemEdit.setFont(new java.awt.Font("Dialog", 0, 12));
    cancelButton.setText("Cancel");
    cancelButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        cancelButton_actionPerformed(e);
      }
    });
    jLabel3.setMaximumSize(new Dimension(100, 17));
    jLabel3.setMinimumSize(new Dimension(100, 17));
    jLabel3.setPreferredSize(new Dimension(100, 17));
    jLabel3.setHorizontalAlignment(SwingConstants.TRAILING);
    jLabel3.setText("Concept form:");
    formItemEdit.setFont(new java.awt.Font("Dialog", 0, 12));
    formItemEdit.setPreferredSize(new Dimension(200, 21));
    priorityItemPanel.setLayout(flowLayout2);
    priorityPanel2.setLayout(borderLayout5);
    priorityPanel3.setLayout(borderLayout6);
    priorityButtonPanel.setLayout(flowLayout3);
    upBtn.setText("Move up");
    upBtn.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        upBtn_actionPerformed(e);
      }
    });
    removeBtn.setText("Remove");
    removeBtn.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        removeBtn_actionPerformed(e);
      }
    });
    addBtn.setText("Add");
    addBtn.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        addBtn_actionPerformed(e);
      }
    });
    downBtn.setText("Move down");
    downBtn.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        downBtn_actionPerformed(e);
      }
    });
    priorityListPanel.setLayout(flowLayout4);
    languageScrollPane.setBorder(langItemEdit.getBorder());
    languageScrollPane.setPreferredSize(new Dimension(200, 100));
    formScrollPane.setBorder(formItemEdit.getBorder());
    formScrollPane.setPreferredSize(new Dimension(200, 100));
    jLabel4.setMaximumSize(new Dimension(100, 17));
    jLabel4.setMinimumSize(new Dimension(100, 17));
    jLabel4.setPreferredSize(new Dimension(100, 17));
    jLabel4.setText("Target language:");
    jLabel5.setText("Concept form:");
    jLabel5.setHorizontalAlignment(SwingConstants.TRAILING);
    jLabel5.setPreferredSize(new Dimension(100, 17));
    jLabel5.setMinimumSize(new Dimension(100, 17));
    jLabel5.setMaximumSize(new Dimension(100, 17));
    jLabel6.setMaximumSize(new Dimension(900, 17));
    jLabel6.setMinimumSize(new Dimension(3, 17));
    jLabel6.setPreferredSize(new Dimension(100, 17));
    languageList.addListSelectionListener(new javax.swing.event.ListSelectionListener()
    {
      public void valueChanged(ListSelectionEvent e)
      {
        languageList_valueChanged(e);
      }
    });
    formList.addListSelectionListener(new javax.swing.event.ListSelectionListener()
    {
      public void valueChanged(ListSelectionEvent e)
      {
        formList_valueChanged(e);
      }
    });
    priorityListPanel.add(jLabel4, null);
    getContentPane().add(panel1);
    panel1.add(mainPanel,  BorderLayout.CENTER);
    panel1.add(buttonPanel,  BorderLayout.SOUTH);
    buttonPanel.add(okButton, null);
    buttonPanel.add(cancelButton, null);
    mainPanel.add(targetLangPanel, BorderLayout.NORTH);
    mainPanel.add(mainPanel2,  BorderLayout.CENTER);
    mainPanel2.add(priorityPanel,  BorderLayout.CENTER);
    targetLangPanel.add(jLabel1, null);
    targetLangPanel.add(langEdit, null);
    targetLangPanel.add(jLabel6, null);
    targetLangPanel.add(loadBtn, null);
    priorityPanel.add(priorityItemPanel,  BorderLayout.NORTH);
    priorityPanel.add(priorityPanel2,  BorderLayout.CENTER);
    priorityItemPanel.add(jLabel2, null);
    priorityItemPanel.add(langItemEdit, null);
    priorityItemPanel.add(jLabel3, null);
    priorityItemPanel.add(formItemEdit, null);
    priorityPanel2.add(priorityButtonPanel, BorderLayout.NORTH);
    priorityPanel2.add(priorityPanel3, BorderLayout.CENTER);
    priorityPanel3.add(priorityListPanel, BorderLayout.NORTH);
    priorityButtonPanel.add(addBtn, null);
    priorityButtonPanel.add(removeBtn, null);
    priorityButtonPanel.add(upBtn, null);
    priorityButtonPanel.add(downBtn, null);
    priorityListPanel.add(languageScrollPane, null);
    priorityListPanel.add(jLabel5, null);
    priorityListPanel.add(formScrollPane, null);
    languageScrollPane.getViewport().add(languageList, null);
    formScrollPane.getViewport().add(formList, null);
  } //jbInit

  /* Reads properties from the concept, if any. */
  private void readProperties()
  {
    if (model_ != null)
    {
      langEdit.setText(model_.getTargetLanguage());
      readTargetLanguages();
    }
  } //readProperties

  /* Sets properties to the concept, if any. */
  private void setProperties()
  {
    model_.setTargetLanguage(langEdit.getText());
  } //setProperties

  /* Reads the target languages for which translation priorities are set. */
  private void readTargetLanguages()
  {
    String selected;
    String language;
    Vector langVec = new Vector();

    // save selected value
    selected = (String) languageList.getSelectedValue();
    // empty list
    languageList.removeAll();
    // fill list with target languages
    Enumeration languages = model_.getTransLanguages();
    while (languages.hasMoreElements())
    {
      language = (String) languages.nextElement();
      langVec.add(language);
    }
    languageList.setListData(langVec);
    // restore selected value
    if (selected != null)
    {
      languageList.setSelectedValue(selected, true);
    }
  } //readTargetLanguages

  /* Reads the concept forms priority list given a target language. */
  private void readConceptForms(String language)
  {
    String selected;
    String form;
    Vector formVec = new Vector();

    // save selected value
    selected = (String) formList.getSelectedValue();
    // empty list
    formList.removeAll();
    // fill list with concept forms
    Enumeration forms = model_.getTransPriorities(language);
    while (forms.hasMoreElements())
    {
      form = (String) forms.nextElement();
      formVec.add(form);
    }
    formList.setListData(formVec);
    // restore selected value
    if (selected != null)
    {
      formList.setSelectedValue(selected, true);
    }
  } //readConceptForms
  
  /**
   * Copies the model properties from one to another model.
   * @param from - the model to copy from
   * @param to - the model to copy to
   */
  private void copyProperties(Model from, Model to)
  {
    to.setTargetLanguage(from.getTargetLanguage());
    to.clearTransPriorities();
    to.copyTransPriorities(from);
  } //copyProperties

  /* Saves properties and closes window. */
  void okButton_actionPerformed(ActionEvent e)
  {
    setProperties();
    dispose();
  } //okButton_actionPerformed

  /* Closes window. */
  void cancelButton_actionPerformed(ActionEvent e)
  {
    dispose();
  } //cancelButton_actionPerformed

  /* Adds a translation priority to the model. */
  void addBtn_actionPerformed(ActionEvent e)
  {
    String language = langItemEdit.getText();
    String form = formItemEdit.getText();

    // check if priority item ok
    if (language.length() == 0)
    {
      reportError("Target language must be set", "Error", null);
      return;
    }
    if (form.length() == 0)
    {
      reportError("Concept form must be set", "Error", null);
      return;
    }
    model_.addTransPriority(language, form);
    readTargetLanguages();
    languageList.setSelectedValue(language, true);
    formList.setSelectedValue(form, true);
  } //addBtn_actionPerformed

  /* Updates the formList. */
  void languageList_valueChanged(ListSelectionEvent e)
  {
    String language = (String) languageList.getSelectedValue();
    readConceptForms(language);
  } //languageList_valueChanged

  /* Updates the language and form edit boxes. */
  void formList_valueChanged(ListSelectionEvent e)
  {
    String language = (String) languageList.getSelectedValue();
    String form = (String) formList.getSelectedValue();
    if (language != null && form != null)
    {
      langItemEdit.setText(language);
      formItemEdit.setText(form);
    }
  } //formList_valueChanged

  /* Removes the selected translation priority from the model. */
  void removeBtn_actionPerformed(ActionEvent e)
  {
    String language = (String) languageList.getSelectedValue();
    String form = (String) formList.getSelectedValue();
    if (language != null && form != null)
    {
      model_.removeTransPriority(language, form);
      readTargetLanguages();
    }
  } //removeBtn_actionPerformed

  /* Moves the selected translation priority up in the list. */
  void upBtn_actionPerformed(ActionEvent e)
  {
    String language = (String) languageList.getSelectedValue();
    String form = (String) formList.getSelectedValue();
    int index = formList.getSelectedIndex();

    if (index < 1 || language == null)
      return;

    model_.removeTransPriority(language, form);
    model_.insertTransPriority(language, form, index - 1);
    readConceptForms(language);
    formList.setSelectedIndex(index - 1);
  } //upBtn_actionPerformed

  /* Moves the selected translation priority down in the list. */
  void downBtn_actionPerformed(ActionEvent e)
  {
    String language = (String) languageList.getSelectedValue();
    String form = (String) formList.getSelectedValue();
    int index = formList.getSelectedIndex();

    if ((index < 0) || (index >= formList.getModel().getSize() - 1) || language == null)
      return;

    model_.removeTransPriority(language, form);
    model_.insertTransPriority(language, form, index + 1);
    readConceptForms(language);
    formList.setSelectedIndex(index + 1);
  } //downBtn_actionPerformed

  /* Opens a load model dialog window and reads model properties. */
  void loadBtn_actionPerformed(ActionEvent e)
  {
    JFileChooser openDlg = new JFileChooser("./models");
    DirFileFilter filter = new DirFileFilter(".xml");
    try
    {
      // filter files on .xml
      openDlg.setFileFilter(filter);
      openDlg.setDialogTitle("Open model");
      if (openDlg.showOpenDialog(this) != JFileChooser.APPROVE_OPTION)
        return;
      File file = openDlg.getSelectedFile();
      // filename chosen - open this file
      String uri = file.getAbsolutePath();
      try
      {
        String name = file.getName();
        if (name.endsWith(".xml"))
          name = name.substring(0, name.length() - 4);
        Model model = new Model(uri, name);
        // model loaded, copy properties and reread
        copyProperties(model, model_);
        readProperties();
      }
      catch (Exception exc)
      {
        reportError("There was an error parsing the XML file: " +
          exc.getMessage(), "Error", null);
      }
    }
    catch (Exception exc)
    {
      reportError("It appears your VM does not allow file loading: " +
        exc.getMessage(), "Error", null);
    }
  } //okButton_actionPerformed

  //////////////////////////////////////////////////////////////////////////////
  // Public methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Shows dialog window.
   */
  public void show()
  {
    readProperties();
    super.show();
  } //show
} //class ModelPropertiesDialog
