/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002, 2003
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License, version 2 as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * (See the file "COPYING" that is included with this source distribution.)
 */

package EDU.utwente.CoCompose.gui;

import EDU.auburn.VGJ.graph.Node;
import EDU.utwente.CoCompose.lang.Model;
import EDU.utwente.CoCompose.lang.AbstractConcept;
import java.awt.Frame;
import java.awt.AWTEvent;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Input dialog for searching for concepts in a model.
 *
 * @author Dennis Wagelaar
 * @version 0.1.10, 22 July 2003
 */

public class SearchDialog extends CoComposeDialog
{
  //////////////////////////////////////////////////////////////////////////////
  // Attributes
  //////////////////////////////////////////////////////////////////////////////

  private ModelCanvas modelCanvas_;
  ImageIcon searchIcon = new ImageIcon("images/zoom.gif");
  ImageIcon closeIcon = new ImageIcon("images/door.gif");
  JPanel buttonPanel = new JPanel();
  JButton searchBtn = new JButton();
  JButton closeBtn = new JButton();
  JPanel topPanel = new JPanel();
  JLabel jLabel1 = new JLabel();
  JPanel namePanel = new JPanel();
  JTextField textName = new JTextField();

  //////////////////////////////////////////////////////////////////////////////
  // Constructors
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Creates new SearchDialog
   * @param frame The parent frame.
   * @param modal Shows the dialog in modal mode when true.
   * @param modelCanvas The model canvas to search in.
   */
  public SearchDialog(Frame frame, boolean modal, ModelCanvas modelCanvas)
  {
    super(frame, "Search", modal);
    modelCanvas_ = modelCanvas;
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try
    {
      jbInit();
      pack();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  } //SearchDialog(Frame frame, boolean modal)

  /**
   * Constructs a SearchDialog.
   * This constructor is only for the JBuilder Designer, do not use it!
   */
  public SearchDialog()
  {
    this(null, false, null);
  } //SearchDialog()

  //////////////////////////////////////////////////////////////////////////////
  // Private methods
  //////////////////////////////////////////////////////////////////////////////

  /* JBuilder Component initialization. */
  private void jbInit() throws Exception
  {
    jLabel1.setText("Element name:");
    jLabel1.setMinimumSize(new java.awt.Dimension(100,22));
    jLabel1.setMaximumSize(new java.awt.Dimension(100,22));
    jLabel1.setPreferredSize(new java.awt.Dimension(150, 22));
    textName.setText("");
    textName.setSize(new java.awt.Dimension(200,26));
    textName.setMinimumSize(new java.awt.Dimension(100, 26));
    textName.setPreferredSize(new java.awt.Dimension(250, 26));
    getContentPane().add(topPanel, java.awt.BorderLayout.CENTER);
    getContentPane().add(buttonPanel, java.awt.BorderLayout.SOUTH);
    searchBtn.setText("Search");
    searchBtn.setIcon(searchIcon);
    buttonPanel.add(searchBtn);
    buttonPanel.add(closeBtn);
    closeBtn.setText("Close");
    closeBtn.setIcon(searchIcon);
    topPanel.setLayout(new java.awt.BorderLayout());
    topPanel.add(namePanel, java.awt.BorderLayout.NORTH);
    setBounds(new java.awt.Rectangle(0,0,485,156));
    closeBtn.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){closeBtnActionPerformed(e);}});
    searchBtn.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){searchBtnActionPerformed(e);}});
    namePanel.add(jLabel1);
    namePanel.add(textName);
  } //jbInit

  /* Closes dialog. */
  void closeBtnActionPerformed(ActionEvent e)
  {
    dispose();
  } //closeBtnActionPerformed
  
  /* Starts a search in the model. */
  void searchBtnActionPerformed(ActionEvent e)
  {
    if (modelCanvas_ == null)
      return;
    Model model = modelCanvas_.getModel();
    if (model == null)
      return;

    for (Node node = model.firstFreeNode(); node != null;
      node = model.nextFreeNode(node))
    {
      if (node instanceof EDU.utwente.CoCompose.lang.AbstractConcept)
      {
        AbstractConcept concept = (AbstractConcept) node;
        String conceptName = concept.getName().toLowerCase();
        if (conceptName.indexOf(textName.getText().toLowerCase()) > -1)
        {
          concept.setFillColor(java.awt.Color.blue);
          modelCanvas_.selectNode(concept.getId());
        }
        else
        {
          concept.setFillColor(concept.getDefaultFillColor());
          concept.setSelected(false);
        }
      }
    } //for
    modelCanvas_.update(false);
  } //searchBtnActionPerformed
} //class SearchDialog
