/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002, 2003
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package EDU.utwente.CoCompose.gui;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.text.*;
import javax.swing.text.html.*;
import java.io.IOException;
import java.net.URL;
import javax.swing.event.*;

/**
 * A text viewing dialog for displaying text contents like source code.
 *
 * @author Dennis Wagelaar
 * @version 0.1.10, 22 July 2003
 */

public class TextViewDialog extends CoComposeDialog
{
  //////////////////////////////////////////////////////////////////////////////
  // Attributes
  //////////////////////////////////////////////////////////////////////////////

  String header = "<HTML><BODY><FONT Face=\"Dialog\" Size=\"2\">";
  String footer = "</FONT></BODY></HTML>";
  JPanel panel1 = new JPanel();
  BorderLayout borderLayout1 = new BorderLayout();
  JButton okBtn = new JButton();
  JPanel jPanel3 = new JPanel();
  JPanel jPanel1 = new JPanel();
  FlowLayout flowLayout1 = new FlowLayout();
  BorderLayout borderLayout2 = new BorderLayout();
  JScrollPane jScrollPane1 = new JScrollPane();
  JEditorPane textArea = new JTextPane();

  //////////////////////////////////////////////////////////////////////////////
  // Constructors
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs a TextViewDialog.
   *
   * @param frame The parent frame.
   * @param title The dialog title.
   * @param modal Shows the dialog in modal mode when true.
   */
  public TextViewDialog(Frame frame, String title, boolean modal)
  {
    super(frame, title, modal);
    try
    {
      jbInit();
      pack();
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  } //TextViewDialog(Frame frame, String title, boolean modal)

  /**
   * Constructs a TextViewDialog.
   * This constructor is only for the JBuilder Designer, do not use it!
   */
  public TextViewDialog()
  {
    this(null, "", false);
  }

  //////////////////////////////////////////////////////////////////////////////
  // Private methods
  //////////////////////////////////////////////////////////////////////////////

  /* JBuilder Component initialization. */
  void jbInit() throws Exception
  {
    panel1.setLayout(borderLayout1);
    okBtn.setText("Close");
    okBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        okBtn_actionPerformed(e);
      }
    });
    jPanel1.setLayout(borderLayout2);
    jPanel3.setLayout(flowLayout1);
    borderLayout2.setHgap(5);
    borderLayout2.setVgap(5);
    panel1.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEtchedBorder(Color.white,new java.awt.Color(148, 145, 140)),BorderFactory.createEmptyBorder(5,5,5,5)));
    panel1.setPreferredSize(new Dimension(800, 600));
    borderLayout1.setHgap(5);
    borderLayout1.setVgap(5);
    textArea.setBackground(getBackground());
    textArea.setEditable(false);
    textArea.addHyperlinkListener(new javax.swing.event.HyperlinkListener()
    {
      public void hyperlinkUpdate(HyperlinkEvent e)
      {
        textArea_hyperlinkUpdate(e);
      }
    });
    jScrollPane1.setBorder(BorderFactory.createLoweredBevelBorder());
    getContentPane().add(panel1);
    panel1.add(jPanel3, BorderLayout.SOUTH);
    jPanel3.add(okBtn, null);
    panel1.add(jPanel1, BorderLayout.CENTER);
    jPanel1.add(jScrollPane1, BorderLayout.CENTER);
    jScrollPane1.getViewport().add(textArea, null);
  } //jbInit

  /* Closes the window. */
  void okBtn_actionPerformed(ActionEvent e)
  {
    dispose();
  } //okBtn_actionPerformed

  /* Handles hyperlinks. */
  void textArea_hyperlinkUpdate(HyperlinkEvent e)
  {
    if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
    {
      try
      {
        if (e instanceof HTMLFrameHyperlinkEvent)
        {
          HTMLDocument doc = (HTMLDocument) textArea.getDocument();
          doc.processHTMLFrameHyperlinkEvent((HTMLFrameHyperlinkEvent) e);
        }
        else
        {
          try
          {
            textArea.setPage(e.getURL());
          }
          catch (Exception exc)
          {
            textArea.setText(header + "Error: could not open " +
              exc.getMessage() + footer);
          }
        } //if e instanceof
      } //try
      catch (Exception exc)
      { // display update can be slow, so ignore errors caused by fast clicking
      }
    } //if e.getEventType
  } //textArea_hyperlinkUpdate

  //////////////////////////////////////////////////////////////////////////////
  // Public methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Sets the dialog text.
   *
   * @param type The MIME content type.
   * @param text The dialog text.
   */
  public void setText(String type, String text)
  {
    textArea.setContentType(type);
    textArea.setText(text);
    // scroll to top
    textArea.setCaretPosition(0);
  } //setText

  /**
   * Sets the current URL being displayed.
   *
   * @param url The URL to open.
   * @throws IOException if the page could not be displayed.
   */
  public void setPage(URL url) throws IOException
  {
    textArea.setPage(url);
    // scroll to top
    textArea.setCaretPosition(0);
  } //setPage

  /**
   * Appends text to the dialog window.
   *
   * @param text The dialog text to append.
   */
  public void appendText(String text)
  {
    Document doc = textArea.getDocument();

    try
    {
      doc.insertString(doc.getLength(), text, null);
    }
    catch (BadLocationException e)
    { // this should never happen
      e.printStackTrace();
    }
  } //addText
} //class TextViewDialog
