/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002, 2003
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package EDU.utwente.CoCompose.util;

import EDU.utwente.CoCompose.lang.AbstractConcept;

/**
 * Exception class for CoCompose model related exceptions.
 *
 * @author Dennis Wagelaar
 * @version 0.1.15, 20 October 2003
 */

public class CoComposeException extends Exception
{
  //////////////////////////////////////////////////////////////////////////////
  // Attributes
  //////////////////////////////////////////////////////////////////////////////

  // The subject of the exception.
  private AbstractConcept concept_ = null;
  // The type of exception (Error, Warning or Info). Default is Error.
  private String type_ = "Error";

  //////////////////////////////////////////////////////////////////////////////
  // Constructors
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs a CoComposeException
   */
  public CoComposeException()
  {
    super();
  } //CoComposeException()

  /**
   * Constructs a CoComposeException
   * @param s - the error message
   */
  public CoComposeException(String s)
  {
    super(s);
  } //CoComposeException()

  /**
   * Constructs a CoComposeException
   * @param s - the error message
   * @param concept - the exception subject
   */
  public CoComposeException(String s, AbstractConcept concept)
  {
    super(s);
    concept_ = concept;
  } //CoComposeException()

  /**
   * Constructs a CoComposeException.
   *
   * @param s - the error message
   * @param concept - the exception subject
   * @param type - the error type
   */
  public CoComposeException(String s, AbstractConcept concept, String type)
  {
    super(s);
    concept_ = concept;
    type_ = type;
  } //CoComposeException()
  
  /**
   * Constructs a CoComposeException
   * @param s - the error message
   * @param from - the exception to copy
   */
  public CoComposeException(String s, CoComposeException from)
  {
    this(s, from.getConcept(), from.getType());
  }

  //////////////////////////////////////////////////////////////////////////////
  // Public methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Gets the subject concept of the exception.
   *
   * @return The subject concept.
   */
  public AbstractConcept getConcept()
  {
    return concept_;
  } //getConcept

  /**
   * Gets the type of the exception.
   *
   * @return The exception type.
   */
  public String getType()
  {
    return type_;
  } //getType
} //class CoComposeException
