/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package EDU.utwente.CoCompose.util;

/**
 * Represents a unique identifier realm.
 *
 * @author Dennis Wagelaar
 * @version 0.0.10, 13 March 2002
 */

public interface UniqueRealm
{
  /**
   * Checks whether the given value is unique within the realm.
   *
   * @param value The value to check.
   * @return True if and only if the value is unique within the realm.
   */
  public boolean isUnique(String value);

  /**
   * Registers a new value in the realm.
   *
   * @param value The value to register.
   */
  public void register(String value);

  /**
   * Unregisters a value from the realm.
   *
   * @param value The value to unregister.
   */
  public void unRegister(String value);
  /**
   * Gets the name of the unique realm.
   *
   * @return The name of the unique realm.
   */
  public String getName();
} //interface UniqueRealm
